//============================================================================
// Name        : MantyBrickPi.cpp
// Description : Example code for Manty/RaspberryPi/BrickPi
// Author      : robotsquare.com
// Version     : v3.14159265359^0.1
// Copyright   : This code is free to use; Hack away and create a Scratch or Python version!
// Disclaimer  : Use at your own risk; Send robots my way; Kindly send complaints to /dev/null
// Info        : Be sure to do the Dexter Industries tutorials before you try this code!
//============================================================================

#include <iostream>
using namespace std;
/////////////////////



#include <stdio.h>
#include <math.h>

// You should add the rt library for timing.
// (See the tutorials from Dexter Industries to do this for compiling from the command line.)
// To do it in Eclipse for your cross compiler:
// 1. Go to: Project Properties > C/C++ General > Paths and Symbols
// 2. Go to the 'Libraries' tab.
// 3. Click 'Add'
// 4. In File, type: rt
// 5. Check: 'Add to all configurations'
// 6. Click OK.

#include <time.h>
#include <unistd.h>
#include "tick.h"

// (See the tutorials from Dexter Industries if you're compiling on the BrickPi itself!)
// For cross compiling, I've simply placed the following file (and its .c counterpart) in my source directory
// I'm not using the LEDs, so I need just the serial library from WiringPi
#include "wiringPi_wiringSerial.h"
// The following line needs to be added in wiringPi_wiringSerial.c:
//     case 500000:	myBaud = B500000 ; break ;
// You should add it after:
//     case 230400:	myBaud = B230400 ; break ;
//Get the serial libraries from here:
// http://wiringpi.com/
// https://git.drogon.net/?p=wiringPi


// I've had to fix some things to make BrickPi.h compile. I believe a new version is being made - go Matt!
#include "BrickPi.h"
#include <linux/i2c-dev.h>
#include <fcntl.h>

int result;

const int Touch = 1;   // Touch Sensor on port 2
const int Left  = 3;   // Left  motor on D
const int Right = 0;   // Right motor on A
const int Head  = 2;   // Head  motor on C
const int Distance = 2;// US Sensor on port 3
const int down = 0;
const int up = 1;

////////////////////
// Milisecond version of usleep
////////////////////

void msleep(int msec){
	usleep(1000*msec);
}

void do_update(){
	////////////////////
	// Run
	// BrickPiUpdateValues()
	// until success
	////////////////////
	  do{
		  result = BrickPiUpdateValues();
		  msleep(10);
		  //cout << "attempt" << endl;
	  }
	  while(result != 0);
	  //cout << "success!"  << endl;
}

int main() {
	  ////////////////////
	  // Hello, World!
	  ////////////////////
	  cout << "----------------" << endl
	       << "Hello, Brick Pi!" << endl
	       << "----------------" << endl;
	  ////////////////////
	  // Set up
	  ////////////////////
	  ClearTick();
	  result = BrickPiSetup();
	  if(result)
	    return 0;

	  BrickPi.Address[0] = 1;
	  BrickPi.Address[1] = 2;
	  BrickPi.MotorEnable[Right] = 1;
	  BrickPi.MotorEnable[Left] = 1;
	  BrickPi.MotorEnable[Head] = 1;
	  BrickPi.SensorType[Touch] = TYPE_SENSOR_TOUCH;
	  BrickPi.SensorType[Distance] = TYPE_SENSOR_ULTRASONIC_CONT;

	  result = BrickPiSetupSensors();
	  if(result)
	    return 0;

	  do_update();

	  int head_direction = up;
	  int difference = 0;
	  const int base_speed = 150;
	  const float gain = 3;

	  ////////////////////
	  // Reset Head
	  ////////////////////

	  //Put head down from any position
	  for(int j = 0; j < 150; j++){
			BrickPi.MotorSpeed[Head] = 70;
		    do_update();
	  }
	  //Make head down the zero position for the encoder
	  int start_head = BrickPi.Encoder[Head];
	  BrickPi.MotorSpeed[Head] = 0;

	  ////////////////////
	  // Keep walking and avoid obstacles
	  ////////////////////

	  while(true){

		  ////////////////////
		  // Sync Legs
		  ////////////////////

		  //Move "randomly" to make sure the Touch Sensor isn't pressed
		  do{
			  while(BrickPi.Sensor[Touch] == 1){
					BrickPi.MotorSpeed[Right]=150;
					BrickPi.MotorSpeed[Left]=150;
					do_update();
			  }
			  BrickPi.MotorSpeed[Right]= 0;
			  BrickPi.MotorSpeed[Left]=  0;
			  msleep(500);
			  do_update();
		  }while(BrickPi.Sensor[Touch] == 1);

		  //Turn Right until Touch Sensor pressed
		  while(BrickPi.Sensor[Touch] == 0){
				BrickPi.MotorSpeed[Right]=150;
				do_update();
		  }
		  //Turn Right until Touch Sensor released
		  while(BrickPi.Sensor[Touch] == 1){
				BrickPi.MotorSpeed[Right]=150;
				do_update();
		  }
		  int start_right = BrickPi.Encoder[Right];
		  //Turn Right 180 degrees further
		  while((BrickPi.Encoder[Right] - start_right) < 360){
				BrickPi.MotorSpeed[Right]=150;
				do_update();
		  }
		  //Make this the zero position for right
		  start_right = BrickPi.Encoder[Right];
		  BrickPi.MotorSpeed[Right]=0;
		  do_update();

		  //Turn Left until Touch Sensor pressed
		  while(BrickPi.Sensor[Touch] == 0){
				BrickPi.MotorSpeed[Left]=100;
				do_update();
		  }
		  //Turn Left until Touch Sensor released
		  while(BrickPi.Sensor[Touch] == 1){
				BrickPi.MotorSpeed[Left]=100;
				do_update();
		  }
		  //Make this the zero position for left
		  int start_left = BrickPi.Encoder[Left];
		  BrickPi.MotorSpeed[Left]=0;
		  do_update();

		  ////////////////////
		  // Walk until object detected
		  ////////////////////

		  //Head must be up & object must be detected to break out of the walk loop
		  while(!(BrickPi.Sensor[Distance] < 20 && BrickPi.Encoder[Head] < -150)){

			  //Get difference between left and right legs
			  difference = (BrickPi.Encoder[Right] - start_right) - (BrickPi.Encoder[Left] - start_left);

			  //Run both motors at base speed, and compensate for difference to keep legs in sync
			  BrickPi.MotorSpeed[Left] = base_speed + difference*gain;
			  BrickPi.MotorSpeed[Right] = base_speed - difference*gain;

			  //Set the speed of the head motor
			  //Negative speed for up; Positive for down
			  if(head_direction == up){
				  if((BrickPi.Encoder[Head] - start_head) < -240){
					  head_direction = down;
				  }
				  else{
					  BrickPi.MotorSpeed[Head] = -100;
				  }
			  }
			  else{
				  if((BrickPi.Encoder[Head] - start_head) > -120){
					  head_direction = up;
				  }
				  else{
					  BrickPi.MotorSpeed[Head] = 30;
				  }
			  }
			  do_update();

		  }
		  //cout << "obstacle! " << BrickPi.Sensor[Distance] << endl;

		  BrickPi.MotorSpeed[Left] =  0;
		  BrickPi.MotorSpeed[Right] = 0;
		  //Put head down for 'sad' expression
		  for(int j = 0; j < 100; j++){
				BrickPi.MotorSpeed[Head] = 40;
			    do_update();
		  }
		  head_direction = up;
		  BrickPi.MotorSpeed[Head] = 0;

		  //Turn for roughly 2.5 seconds
		  for(int j = 0; j < 250; j++){
			  BrickPi.MotorSpeed[Left] = base_speed;
			  BrickPi.MotorSpeed[Right] = -base_speed;
			  do_update();
		  }
		  BrickPi.MotorSpeed[Left] =  0;
		  BrickPi.MotorSpeed[Right] = 0;

	  }// End of while(true)
	return 0;
}
